'use strict';
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Module dependencies.
 */
var bind_all_1 = __importDefault(require("bind-all"));
var clone_1 = __importDefault(require("./utils/clone"));
var cookie_1 = __importDefault(require("@segment/cookie"));
var debug_1 = __importDefault(require("debug"));
var defaults_1 = __importDefault(require("@ndhoule/defaults"));
var top_domain_1 = __importDefault(require("@segment/top-domain"));
// eslint-disable-next-line new-cap
var debug = debug_1.default('analytics.js:cookie');
/**
 * Initialize a new `Cookie` with `options`.
 *
 * @param {Object} options
 */
var Cookie = function (options) {
    _this.options(options);
};
Cookie.prototype.options = function (options) {
    if (arguments.length === 0)
        return this._options;
    options = options || {};
    var domain = '.' + top_domain_1.default(window.location.href);
    if (domain === '.')
        domain = null;
    this._options = defaults_1.default(options, {
        // default to a year
        maxage: 31536000000,
        path: '/',
        domain: domain,
        sameSite: 'Lax'
    });
    // http://curl.haxx.se/rfc/cookie_spec.html
    // https://publicsuffix.org/list/effective_tld_names.dat
    //
    // try setting a dummy cookie with the options
    // if the cookie isn't set, it probably means
    // that the domain is on the public suffix list
    // like myapp.herokuapp.com or localhost / ip.
    this.set('ajs:test', true);
    if (!this.get('ajs:test')) {
        debug('fallback to domain=null');
        this._options.domain = null;
    }
    this.remove('ajs:test');
};
/**
 * Set a `key` and `value` in our cookie.
 *
 * @param {String} key
 * @param {Object} value
 * @return {Boolean} saved
 */
Cookie.prototype.set = function (key, value) {
    try {
        value = window.JSON.stringify(value);
        cookie_1.default(key, value === 'null' ? null : value, clone_1.default(this._options));
        return true;
    }
    catch (e) {
        return false;
    }
};
/**
 * Get a value from our cookie by `key`.
 *
 * @param {String} key
 * @return {Object} value
 */
Cookie.prototype.get = function (key) {
    try {
        var value = cookie_1.default(key);
        value = value ? window.JSON.parse(value) : null;
        return value;
    }
    catch (e) {
        return null;
    }
};
/**
 * Remove a value from our cookie by `key`.
 *
 * @param {String} key
 * @return {Boolean} removed
 */
Cookie.prototype.remove = function (key) {
    try {
        cookie_1.default(key, null, clone_1.default(this._options));
        return true;
    }
    catch (e) {
        return false;
    }
};
/**
 * Expose the cookie singleton.
 */
module.exports = bind_all_1.default(new Cookie());
/**
 * Expose the `Cookie` constructor.
 */
module.exports.Cookie = Cookie;
